# Load necessary packages ----

# If you don't have the packages below installed, install them with
install.packages(c("ape", "picante", "phytools"))

# Load the packages
library(ape)
library(picante)
library(phytools)


# Phylogenetic tree formats ----

# Create a simple tree with no branch lengths info as a character string in the
# Newick format.
tree_newick <- "((taxaA, taxaB), (taxaC, taxaD));"

# Read this tree with the read.tree function. It gets transformed into a phylo
# object. 
tree1 <- read.tree(text = tree_newick)

# Plot the tree we just created
plot(tree1)

# Have a look at what data structure the phylo object tree1 has and what it 
# contains.
str(tree1)

# Do the same as for the tree above, but with a tree that includes branch 
# lengths and names for the inner nodes (not just the species at the external
# nodes).
tree_newick <- 
  "((taxaA:0.5, taxaB:0.5)nodeA:1.5, (taxaC:1, taxaD:1)nodeB:1)nodeX;"
tree2 <- read.tree(text = tree_newick, show.node.label = TRUE)
plot(tree2)
str(tree2)

# Exercise tree
tree_newick <- 
  "((A:2, B:2)Y:2, ((C:1, D:1)Z:2, (E:1.5, F:1.5)V:1.5)W:1)X;"
tree3 <- read.tree(text = tree_newick, show.node.label = TRUE)
plot(tree3)
str(tree3)


# Phylogenetic independent contrasts ----
# Create a tree with 8 species, the branch lengths are equal, i.e. they are all
# set to 1. 
tree_newick <- 
  "(((A:1, B:1):1, (C:1, D:1):1):1, ((E:1, F:1):1, (G:1, H:1):1):1);"
tree <- read.tree(text = tree_newick)
plot(tree)

# Create two hypotehtical variables x and y. Note: The values are made up so
# that the correlation between the two variables is significant for both the
# standard correlation (or linear regression) and also the correlation based on
# independent contrasts.
x <- c(1, 3, 5, 2, 6, 8, 2, 7)
y <- c(5, 6, 8, 3, 7, 9, 4, 9)

# Calculate the phylogenetic independent contrasts for these variables, using 
# the pic function and the tree we created above, with the pic function from the
# ape package.
xp = pic(x, tree)
yp = pic(y, tree)

# Plot the scatterplots for x and y, and then for their pics. 
plot(x, y, pch = 16)
plot(xp, yp, pch = 16)

# Another way to plot the trait values is with the phylomorphospace function. It
# displays the original trait values, but adds the values for the ancestral 
# species and also adds lines between species and nodes to reflect the 
# underlying topology of the phylogenetic tree of the species. 
phylomorphospace(tree, cbind(x, y),
                 xlab="trait 1",ylab="trait 2")


# If we want to do a simple linear regression on the independent contrasts, we
# need to force the regression slope through the origin, i.e. it will pass 
# through the 0,0 coordinate on our plot.
pic_lm <- lm(yp ~ xp - 1)

# Our linear model is significant, and the slope estimate is 0.98.
summary(pic_lm)

# Display the regression slope in the scatterplot of the independent contrasts. 
plot(xp, yp, pch = 16)
abline(0, pic_lm$coeff)

# Phylogenetic signal in traits ----
library(picante)

# Read in data on some plant traits (and their log transformations)
lhs <- read.table("data/lhs.txt", header = TRUE)
str(lhs)

# Read in a phylogenetic tree that contains those species in the trait data and 
# take care of different formats of the species in names in both data, so that
# trait and phylogenetic data can be matched by their species names.

tree_daphne <- read.tree("data/DaPhnE_01.tre")
plot(tree_daphne)

tree_daphne$tip.label <- gsub("_", " ", tree_daphne$tip.label)

# Show the first few species names of the tree to check that we got the desired 
# change in the species name format, i.e. the same that is used in the trait 
# data with out the "_" between genera and species names. 
head(tree_daphne$tip.label)

# Read in a custom function that can match tree and trait (and community) data
# by species names.
source("data/match_data.R")

# Match tree and trait data by species names.
match_traits_phylo <- match_data(traits = na.omit(lhs), tree = tree_daphne)

# The output is a list with three elements. You can call the trait data via 
# $traits and the tree via $tree from the resulting object. There is also an
# $com element, but it is empty because we didnt give the function any community 
# data to also be matched to the other data by name. 
str(match_traits_phylo)

head(match_traits_phylo$traits)

match_traits_phylo$tree

# For the function we use for the phylosignal calculation, we need to provide
# the traits as single vectors, so we save two of the traits as separate vectors
# under the objects sla and sm (i.e. the traits specific leaf area and seed 
# mass, in their log-transformed version).
sla <- as.matrix(match_traits_phylo$traits)[, 4]
sm <- as.matrix(match_traits_phylo$traits)[, 6]

# Estimate phylogenetic signal with function phylosig from package phytools

# First for the trait seed mass, both for Bloombergs K and Pagel's lambda. 
phylosig(match_traits_phylo$tree, sm, method = "K")

phylosig(match_traits_phylo$tree, sm, method = "lambda")

# Then for the trait sla, both for Bloombergs K and Pagel's lambda. 
phylosig(match_traits_phylo$tree, sla, method = "K")

phylosig(match_traits_phylo$tree, sla, method = "lambda")

# Traitgrams ----

# Because traitgrams for the entire tree and all the trait data would be not 
# very helpful to look at (too many species), we will produce two traitgrams 
# that show the tree and traits for a small set of species, based on two plots
# in a large set of vegetation plot data. 
load("data/mesicmeadows.Rdata")

# We pick the two communities that we want to draw the traitgrams for. 
com1 <- mesicm[158,][, mesicm[158,] != 0]
com2 <- mesicm[21,][, mesicm[21,] != 0]

# Now, we match the trait and tree data to each of those two communities. 
match1 <- match_data(com = com1, traits = na.omit(lhs), tree = tree_daphne)
match2 <- match_data(com = com2, traits = na.omit(lhs), tree = tree_daphne)

# Draw a traitgram for each of the two communities, next to each other, 
# displaying different levels of phylogenetic signal, as demonstrated by
# their actual Bloomberg's K values calculated below. 
par(mfrow = c(1, 2))
traitgram(as.matrix(match1$traits["log_sla"]), multi2di(match1$tree),
          show.names = TRUE)
traitgram(as.matrix(match2$traits["log_sla"]), multi2di(match2$tree),
          show.names = TRUE)

phylosig(match1$tree, match1$traits[, "log_sla"])

phylosig(match2$tree, match2$traits[, "log_sla"])

# Phylogenetic diversity ----

# Match phylogenetic and community data. The community data are the same that we
# used above to show the traitgrams. Here we use all the communities that are in
# the data, all together vegetation 296 plots of mesic meadows distributed 
# across the Czech Republic. 
match_mm <- match_data(com = mesicm, tree = tree_daphne)

# Calculate species pairwise phylogenetic distances for the tree that contains
# now only the species that are in our vegetation plots. 
phylo_dist <- cophenetic(match_mm$tree)

# Show the calculated distances for the first four species in our data.
phylo_dist[1:4, 1:4]

# Show how these species are related on the tree. Note how the distances from
# the previous line correspond to the phylogenetic relationships.
ape::zoom(match_mm$tree, 1:4)

# Calculate phylogenetic diversity. The first measure is one that sums 
# phylogenetic branch length on the tree of the species occurring in the 
# communities (in the first column of the output). It does also calculate the 
# species richness for the samples at the same time (second column).  
mpd_pdfaith <- pd(match_mm$com, match_mm$tree)

# The second measure calculate here takes the mean of all pairwise distances 
# between species that are in the community.
mpd_pd <- mpd(match_mm$com, phylo_dist)

# Plot the pairwise scatterplots between the two phylogenetic diversity indices
# and species richness. It is clear that Faith's phylogenetic diversity is 
# strongly positively related with species richnes, while mpd is not. 
pairs(data.frame(mpd_pd, mpd_pdfaith$PD, mpd_pdfaith$SR))
